#ifndef M_BTREE_H 
#define M_BTREE_H

/*
   --- definitions for arithmetic binary trees

   $Log: btree.h,v $
   Revision 1.2  1997/06/16 18:13:12  kraemer
   *** empty log message ***


*/
                            /* descriptor per dimension */
struct SBTREEDIM {
   unsigned long lFlag;
   long iLB, iHB, iStep;    /* lower and upper bound (C-syntax), stride */
   int  nLB, nHB;           /* length of cLB, cHB */
   const char *cLB, *cHB;   /* l,h-bound strings */
   unsigned long lStride, lElm; /* accum. stride, # elements this dim. */
};
                            /* internal array descriptor */
#define MBTREEDIM 16
struct SBTREEDESCR {
   double *pd; int nDim;
   struct SBTREEDIM sDim[MBTREEDIM];
};
                                   /* status flags */
#define BTREE_SUBLOOP 0x01000000   /* subscript loop given */
#define BTREE_WILD    0x02000000   /* name/subscript contains a wild chard */
#define BTREE_ARRC    0x04000000   /* array given in "C" syntax */
#define BTREE_SUBEXPR 0x08000000   /* subscript might be an expression */

                     /* array descriptor used for parsing */
struct SBTREEARRAY {
   unsigned long lFlag;     /* status flag */
   const char *cName;       /* array base name */
   int   nName;
   char *cFullName;         /* array name with current subscripts */
   int nDim;                /* current dimension */
   unsigned long lElm;                /* # total # of elements */
   struct SBTREEDIM sDim[MBTREEDIM];
};
                     /* a tree node */
struct SBTREE {
   double  dSon[2];  /* holds converted constant values */
   void   *vSon[2];  /* const/var name or ptr. to son */
   char *cOp;        /* operation name */
   int  iOp;         /* internal opcode */
                     /* 0: result, 1/2: left/right operand*/
   struct SBTREEDESCR sBTD[3];
}; 
                   /* an identifier table entry (scalar only)*/
struct SBTREESIDF {
   char *cIdf; double *pIdf;
};

                   /* an identifier table entry (scalar/vector)*/
struct SBTREEVIDF {
   char *cIdf; double *pIdf;
   int ii, nn;     /* stride/element count */
};

                   /* an identifier table entry (general)*/
struct SBTREEDIDF {
   char *cIdf; struct SBTREEDESCR *pBTD;
};

                   /* an operator table entry */
struct SBTREEOP {
   char *cOp; int iOp;
};
                   /* list of internal operation codes */
enum {
   BTREE_NOP,
   BTREE_ADD,
   BTREE_SUB,
   BTREE_MUL,
   BTREE_DIV,
   BTREE_POW,
   BTREE_NEG,
   BTREE_ABS,
   BTREE_EXP,
   BTREE_LOG,
   BTREE_SQRT,
   BTREE_LOG10,
   BTREE_LOG2,
   BTREE_SIN,
   BTREE_COS,
   BTREE_TAN,
   BTREE_ASIN,
   BTREE_ACOS,
   BTREE_ATAN,
   BTREE_SINH,
   BTREE_COSH,
   BTREE_TANH,
   BTREE_ASINH,
   BTREE_ACOSH,
   BTREE_ATANH
};
                            /* action flags */

#define BTREE_RESOLVESIDF   0x00000001
#define BTREE_RESOLVEOP     0x00000002
#define BTREE_EVALUATE      0x00000004
#define BTREE_DIFFERENTIATE 0x00000008
#define BTREE_CLEANUP       0x00000010
#define BTREE_ASMCODE       0x00000020
#define BTREE_RESOLVEVIDF   0x00000100
#define BTREE_RESOLVEDIDF   0x00000400
#define BTREE_TEST          0x00008000
#define BTREE_ERRMSG        0x00004000

struct SBTREE *c2btree( const char *, int, char **, int * );
int btreeprc( void *, void *, int, int );
int btreearr( const char *, int, struct SBTREEARRAY *);

#define ERRBTREE_SYS      -1 
#define ERRBTREE_SYNTAX   -2 
#define ERRBTREE_ID       -3 
#define ERRBTREE_OP       -4 
#define ERRBTREE_INVBTREE -5 
#define ERRBTREE_DOMAIN   -6
#define ERRBTREE_NOBTREE  -7
#define ERRBTREE_ACTIONNOTYET -8
#define ERRBTREE_INVDIM   -9
#define ERRBTREE_INVPTR   -10
#define ERRBTREE_INVNAM   -11
#define ERRBTREE_INVNDIM  -12
#define ERRBTREE_INVSUB   -13
#define ERRBTREE_ARRSIZE  -14
#define ERRBTREE_CONVSUB  -15  /* can't convert subscript */

#endif
