#ifndef DISP_H
#define DISP_H
/*
   === declarations for display subsystem ================================
   $Log: disp.h,v $
   Revision 1.4  2000/04/03 13:13:14  kraemer
   adapted for 2D

   Revision 1.3  1999/08/04 14:14:18  kraemer
   added msg for unimplemented commands

   Revision 1.2  1999/06/18 11:39:34  kraemer
   linux-specials

   Revision 1.1.1.1  1997/07/10 19:17:32  kraemer

*/

#ifndef G_H
#include "g.h"
#endif
#ifndef ANL_H
#include "anl.h"
#endif
#ifndef EVT_H
#include "evt.h"
#endif
#ifndef FIT_H
#include "fit.h"
#endif

#include <pthread.h>

#if defined(NOMORE)
struct SDISPATTR {
   double dWidth;            /* width (mm) */
   double dSize;             /* symbol size (mm) */
   unsigned long lRGB[3];    /* rgb colour composition */
   int iCMode, iLineMode, iSymbol; /* GD-style char mode, line mode */
   char cMarker[4];          /* 'frsb' (fill/rotation/size/type) */
   char cLineMode[3];        /* 'gdb' gap/dash/type (not used) */
   char cFont[2];            /* 'db' distance/type */
};

struct SDISPLINE {           /* line attributes */
   double dSize;             /* symbol size (mm) */
   double dLineWidth;
   unsigned long lRGB[3];    /* rgb colour composition */
   unsigned long lLineType;  /* spline, polygon etc. */
   unsigned long lLineMode;  /* GD-style dashing */
   int iSymbol;
};

struct SDISPTEXT {           /* text attributes */
   double dWidth, dHeight, dAngle, dIncl;
   double dLineWidth;
   unsigned long lRGB[3];    /* rgb colour composition */
   int iCMode;
};
#endif 

#define MDISPATTR 10
                             /* structure to hold data pointers */
struct SDISPDATA {
   int nn;
   double *dBin,                  /* [nn+1] */
          *dSymSize,              /* [nn] */
          *dXY;                   /* [nn][2] */
   double *dX,                    /* [nn] */
          *dY,                    /* [nn] */
          *ddXm, *ddXp,           /* [nn] */
          *ddYm, *ddYp;           /* [nn] */
};
                             /* structure for window bar bounding box */
struct SDISPWBARBOX {
   struct Node ln;
   double dBox[2][2];        /* xmin,xmax,ymin,ymax */
};
/*
   --- live display stuff
*/

#define MDLIVE 4                 /* max # of live displays */

#define DLIVE_ON   0x00000001    /* live display active */
#define DLIVE_RND  0x00000002    /* add random # */
#define DLIVE_COMM 0x00000004    /* comment on display */

struct SDLIVEANL {               /* describes live display analyzer */
   long lDim;                    /* displayed dimension of analyzers - 1 */
   long nDim;                    /* max       dimension of analyzers */
   long lLimAnl[MANLDIM][4];     /* limits of x,y analyzers */
   long lLim   [MANLDIM][4];     /* used limits in x,y */
   struct SANL *pAnl;            /* ptr to header of analyzer */
};

struct SDLIVE {                  /* structure for 1 live display */
   unsigned long lFlag;
   unsigned long lDLPnt,mDLPnt;  /* current, max # of live displays */
   unsigned long lMarker;
   unsigned long lColour;
   unsigned long lRGB[3];
   int iSymbol;
   double dSymSize, dLWidth, dHWidth;
   struct SGH  *pGH;
   struct SDLIVEANL sDLA[2];     /* description of 2 analyzers */
};

struct SDLIVEFRAME {             /* container structure for live display */
   unsigned long lFlag;          /* */
   int iDL, nDL;                 /* current free slot,
                                    max # of live displays */
   int nx,ny;                    /* # of pictures in x,y direction */
   unsigned long lDLPnt, mDLPnt; /* current, max # of live points */
   unsigned short iSeed[4];      /* random seed */
   double dPlotArea[2][2];       /* plot area for 1 live picture */
   double dAxisArea[2][2];       /* axis area for 1 live picture */
   double dMargin[2];
   struct SDLIVE *pDL;           /* [nDL] */
   pthread_mutex_t Mutex;        /* mutex for live display access */
};

struct SDISP {
   double dPlotArea[2][2]; /* total plot area (cm): offx,offy,width,height */
   double dAxisArea[2][2]; /* axis  plot area (cm): offx,offy,width,height */
   double dMargin[2];      /* right/upper margin (cm) for descriptions */
   double dAxisTxtSize;    /* axis text size (cm) */
   double dAnlTxtSize;     /* analyzer description text size (cm) */
   double dWdwTxtSize;     /* window description text size (cm) */
   double dCommTxtSize;
   double dvLim[3][2];       /* logical limits */
   double dFitTextPos[2];    /* descriptive text position (cm), from fit */
   double dTextPos[2];       /* descriptive text position (cm) */
   double dCommPos[2];       /* comment text position (cm) */
                             /* attributes of last displayed analyzer */
   long lAnl, lEvt, nDim, lTyp, lBin;
   int  nDimDisp;            /* 1 or 2 */               
   long lvDim[2];            /* displayed dimension(s) */   
   long lvBin[2];            /* displayed binsize */   
   long lLimAnl[MANLDIM][4], lLimWdw[MANLDIM][4];
   char cAnl[MANLNAME+1],cA0;
   char cEvt[MEVTNAME+1],cE0;

   struct SGH *pGH;          /* display handle */
   struct SFITBLK   *pFitBlk;
   struct SDISPDATA sData;
   unsigned long lStatus;    /* coord type et al. */
   unsigned long ulvScale[3];/* global lin/log/sqrt/recip for x/y/z */
   unsigned long ulvMode [3];/* global auto/fix, dscale for x/y/z */
   unsigned long ul2DRepr;   /* iso/clu/... */
   long lPicType;            /* $I_DPIC
                                (1/2/3/4/5)=(1D/2D/LIVE/DPLOT/DCOPY) */
   char *pcCuts;
   char cDevice[80],cDevice0[2];
   char cWdwName[80],cWdwName0[2];
   char cCom[78],cCom0[2];   /* comment, padding */
   char cLastWdw[20];        /* last used window/condition */
   char cLastPnt[20];        /* last used display point */
   struct List lhWBar;       /* Head of list of window bar bounding boxes */
#if defined(NOMORE)
   struct SDISPLINE sLine;   /* local line attributes */
   struct SDISPTEXT sText;   /* local text attributes */
#endif
                             /* global attributes */
   double dWidth     [MDISPATTR];
   double dSymSize   [MDISPATTR];
   unsigned long lRGB[MDISPATTR][3];
   int iSymbol       [MDISPATTR];
   int iLineMode     [MDISPATTR];
   int iCMode        [MDISPATTR];
                                      /* overlay attribute indices */
   int  iOLColour, nOLColour;         /* # of current/valid entries */
   long lOLColour[MDISPATTR];
   int  iOLLWidth, nOLLWidth;
   long lOLLWidth[MDISPATTR];
   int  iOLLMode, nOLLMode;
   long lOLLMode[MDISPATTR];
   int  iOLMarker, nOLMarker;
   long lOLMarker[MDISPATTR];
   int  iOLPattern, nOLPattern;
   long lOLPattern[MDISPATTR];
   struct SDLIVEFRAME sDLF;
};
/*
   --- action and status flags ------------------------------------------
*/
enum {
   DISP_PIC1D=1, DISP_PIC2D, DISP_PICLIVE, DISP_PICDPLOT, DISP_PICDCOPY,
};
                                      /* axis scales */
enum {
   DISP_LIN=0, DISP_LOG, DISP_SQRT, DISP_RECIP,
};
                                      /* 2D representations */
enum {
   DISP_2DCLU=1, DISP_2DISO, DISP_2DCON, 
};
                                      /* line styles */
enum {
   DISP_NOLINE=0,
   DISP_SOLID, DISP_DOT, DISP_DOTDASH,
   DISP_LONGDASH, DISP_DASH, DISP_LONGDOTDASH,
};

                                      /* markers */
enum {
   DISP_MARK_NONE=0,
   DISP_MARK_DOT, DISP_MARK_PLUS, DISP_MARK_CROSS, DISP_MARK_AST,
   DISP_MARK_CIRCLE, DISP_MARK_SQUARE,
   DISP_MARK_RTRIANGLE, DISP_MARK_LTRIANGLE,
   DISP_MARK_REDGE, DISP_MARK_LEDGE, DISP_MARK_VERT, DISP_MARK_CORNER,
   DISP_MARK_FCIRCLE, DISP_MARK_FSQUARE,
   DISP_MARK_FRTRIANGLE, DISP_MARK_FLTRIANGLE,
   DISP_MARK_LAST,
};
                                       /* text fonts */
enum {
   DISP_FONT_HARD=0,
   DISP_FONT_ROMANSIMPLEX, DISP_FONT_GREEKSIMPLEX,
   DISP_FONT_ROMANCOMPLEX, DISP_FONT_GREEKCOMPLEX,
   DISP_FONT_ITALICCOMPLEX,
   DISP_FONT_ROMANDUPLEX, DISP_FONT_MATH,
};  
/*
   --- global flags
*/
#define DISP_CLIPON    0x00000001      /* clipping on */
#define DISP_SCALESET  0x00000002      /* fixed scale limits set */
#define DISP_FIX       0x00000004      /* fixed scale */
#define DISP_DSCALE    0x00000008      /* DSCALE command given */

#define DISP_ERR       0x00000010      /* error bars active */
                                       /* line type */
#define DISP_SPLINE    0x00000100
#define DISP_LINE      0x00000200
#define DISP_POLYGON   0x00000400
                                       /* coordinate types */
#define DISP_COORDLOG  0x00001000
#define DISP_COORDCM   0x00002000
#define DISP_COORDNORM 0x00004000

#define DISP_COMMON    0x00020000
#define DISP_COMMTEXT  0x00040000
#define DISP_COMMFILE  0x00080000

#define DISP_COPYON    0x00100000      /* copy plotfile active */
#define DISP_LIVEON    0x00200000      /* live display active */

#define DISP_DEBUG     0x80000000      /* global debug */
/*
   --- local flags
*/
#define DWRWDW_STATIC  0x00000001      /* static window */
#define DWRWDW_POINT   0x00000002      /* use point */
#define DWRWDW_NOMARK  0x00000004      /* don't mark in spectrum */

/*
   --- function declarations --------------------------------------------
*/
                                       /* user callable */
int _Dbell( int, int );
int _Dclip( unsigned long, int, double [2], double [2] );
int _Dfpeak();
int _Diline( int, char *, int, int, double, int );
int _Dinit( int, char *, double );
int _Ditext( int, double, double, double, double );
int _Dline( double *, double *, int, unsigned long );
int _Dpnt( double *, double *, unsigned long * );
int _Dtext( double, double, int, char *, unsigned long );

                                       /* command processors */
void _Dattr( long *, long *,
             char *, char *, char *,
             char *, char *, char *,
             char *, char *, char *, char *,
             long *);
void _Dcommt( char *, long *, long *, long *, long * );
void _Dcontr();
void _Dcopy( char *, char *, char *, char *, char *,
             long *,
             char *, char *, char *, char *,
             char *, char *, char *,
             long *, long *, long *,
             long *, long *,
             long *, long *, long *, long * );
void _Ddevice( char *, long *, long *, long *, long * );
void _Dexpan( char *, char *,
              long *, long *, long *,
              long *, long *, long *, long *,
              long *
            );
	    
void _Dfpcmd( long *,
              double *, double *, double *,
	      long *, long *, long *,
	      const char *, 
	      long *, long *, long *,
	      long *   
            );
void _Dgtpnt( char *, char *, long *, long * );
void _Dgtwdw(
             char *, char *,
             long *, long *, long *,
             long *, long *, long *, long *, long *,
             long *
            );
void _Displa( char *, char *,
              long *, long *, long *, long *,
              long *, long *, long *,
              char *,
              long *, long *, long *,
              long *, long *, double *,
              char *, long *,
              long *, long *, long *,
              long *, long *, long *
            );
void _DLcnd ( long *, char *, char *, long *, long *, long * );
void _DLcont( long *, long *, long *, long *, long * );
void _DLfini( long *, long * );
void _DLinit( long *, long *, long *, long *, long *, long *, long * );
void _DLive ( char *, char *, char *, char *, char *,
              long *, long *, long *, long *, long *, long *, long * );
void _DLsum( long * );
void _DLterm( void );

void _Dchan();
void _Dmode( char *,
             long *, long *, long *, long *,
             long *, long *, long *,
             long *, long *, long *
           );
void _Doverl(
             char *, char *,
             double *, double *, double *, double *,
             double *, double *,
             double *, double *,
             char *,
             long *, long *, long *, long *, long *,
             char *, char *,
             char *, char *, char *,
             long *
            );
void _Dcpfk();
void _Dplot();
void _Dpoint( char *, long *, long *, long * );
void _Dscale( char *, char *, long *, long *, long *, long *,
             long *, char *, long *, long *, long * );
void _Dshift();
void _Dsvpnt( char *, double *, long *, long *, long * );
void _Dsavew( char *, char *, double *, long *, long *, long * );
void _Dwrite();
void _Dwindo(
             char *, double *,
             long *, long *, long *,
             long *, long *,
             long *
            );
void _D2disp(
              const char *, const char *,
	      long *, long *, long *, long *, long *,
              const char *, const char *,
	      const char *, const char *,
	      const char *,
	      long *, long *, long *,
	      const char *, long *, long *,
              const char *, const char *,
	      const char *, const char *,
	      double *, double *,
	      const char *,
	      long *,
	      const char *, long *,
	      long *, long *, long *,
	      long *
            );
void _D2cond( const char *, long *, long *, long *);
void _D2exp( long *, long *, long *, long *);
                                             /* internal */
int _DLdo( unsigned long );
int _dspinit( unsigned long );
int _dspterm( unsigned long );
void _dspemsg( char *, int );

int  _dspdtal( long lLim[4], double *,
               struct SDISPDATA *, unsigned long );
void _dspdtfr( struct SDISPDATA *, unsigned long );

int _Dwrwdw( long, long, char *, long [], int, unsigned long );
int _Dwrcom( unsigned long );
int _Dwrnam( char *, char *, unsigned long );
int _Dwrdim( long [][4], long [], int, int,  unsigned long );

int _DFitStore( struct SFITANLXFER *, struct SFITDATAXFER *, unsigned long );
int _DFitError( struct SFITANLXFER *, struct SFITDATAXFER *, unsigned long );
int _DFitWdw( char *, int, struct SFITWDW *, unsigned long );
int _DFitGfxAttr( struct SFITDISPATTR *, struct SGFXATTR *, unsigned long );
int _DFitGfxTxt ( struct List *, unsigned long );

void _Dtest( char *, long *);
/*
   --- error flags ------------------------------------------------------
*/

#define ERRDISP_SYS           -1
#define ERRDISP_NODISP        -2
#define ERRDISP_CMDNOTYET     -3
//#define ERRDISP_PAREXCL       -4
#define ERRDISP_NOCURSOR      -5
#define ERRDISP_POINTOUTSIDE  -6
#define ERRDISP_INIT          -7
#define ERRDISP_LINLOGEXCL    -8
#define ERRDISP_NO1D          -9
#define ERRDISP_INVAXIS      -10
#define ERRDISP_INVANLTYP    -11
#define ERRDISP_INVANLDIM    -12
#define ERRDISP_PARNOTYET    -13
#define ERRDISP_INVSPCREPR   -14
#define ERRDISP_GETSPC       -15
#define ERRDISP_OPNWDW       -16
#define ERRDISP_SCALE        -17
#define ERRDISP_INVSIZE      -18
#define ERRDISP_SYNTAX       -19
#define ERRDISP_INVCOLOUR    -20
#define ERRDISP_INVMARKER    -21
#define ERRDISP_INVFONT      -22
#define ERRDISP_INVLINE      -23
#define ERRDISP_INVMSIZE     -24
#define ERRDISP_INVWIDTH     -25
#define ERRDISP_SPCEMPTY     -26
#define ERRDISP_AXISAREA     -27
#define ERRDISP_INVWDW       -28
#define ERRDISP_PLOTCOPY     -29
#define ERRDISP_PAREXCL      -30
#define ERRDISP_FIT          -31
#define ERRDISP_INVPNTNO     -32
#define ERRDISP_BADADDR      -33
#define ERRDISP_CONV         -34
#define ERRDISP_BADSTRETCH   -35
#define ERRDISP_INVCND       -36
#define ERRDISP_INVPNT       -37
#define ERRDISP_DLINVNUM     -38
#define ERRDISP_DLINVPNT     -39
#define ERRDISP_DLINVTIMER   -40
#define ERRDISP_DLNOLIVEPIC  -41
#define ERRDISP_DLACTIVE     -42
#define ERRDISP_DLINVDIM     -43
#define ERRDISP_DLINVANLDIM  -44
#define ERRDISP_DLINVWDW     -45
#define ERRDISP_DLANLCHK     -46
#define ERRDISP_DLANLDYN     -47
#define ERRDISP_DLCANTLAYOUT -48
#define ERRDISP_MUTEXINIT    -49
#define ERRDISP_MUTEXLOCK    -50
#define ERRDISP_MUTEXUNLOCK  -51
#define ERRDISP_ACHECK       -52
#define ERRDISP_ACND         -53
#define ERRDISP_AGTSPC       -54
#define ERRDISP_NO1DNO2D     -55
#define ERRDISP_BADBINSIZE   -56
#define ERRDISP_REPREXCL     -57
#define ERRDISP_BADCUTS      -58
#define ERRDISP_BADZSCALE    -59
#define ERRDISP_BADMATHDOMAIN -60
#define ERRDISP_NO2D          -61
#define ERRDISP_NOCNDAVAIL    -62
#define ERRDISP_BADCURSOR     -63

#endif
