#ifndef IOIO_H
#define IOIO_H
/*
   Definitions for I/O 
   ===================

   Copyright  (C) 1995-2000 M.Kraemer, GSI Darmstadt,
                             all rights reserved   

   $Log: ioio.h,v $
   Revision 1.4  2001/01/16 13:40:58  kraemer
   UMAC listmode format added

   Revision 1.3  1999/07/30 13:43:03  kraemer
   changed go/halt/pending logic to allow for 'iwait'-command

   Revision 1.2  1999/07/26 19:02:00  kraemer
   various bugs fixed: event defragmentation, error codes, file name keeping

   Revision 1.1  1999/07/17 16:19:04  kraemer
   major overhaul of edas/goosy buffer and swap logic


*/

#include <pthread.h>

#include "u.h"
#include "iorev.h"
#include "ioedas.h"
#include "iogoosy.h"
#include "ioumac.h"
#include "iofh.h"
                                      /* masks */
#define IO_MASKGLOBAL 0xFF000000      /* global (e.g. debug) */
#define IO_MASKFUNC   0x00FF0000      /* function specific flags */
#define IO_MASKQUAL   0x0000FFFF      /* for additional qualifiers */
                                      /* flags */
#define IO_DEBUG      0x80000000      /* general check flag */
#define IO_CHECK      IO_DEBUG        /* general check flag */
#define IO_QUIET      0x10000000      /* suppress error messages */
#define IO_NOMSG      IO_QUIET

                                      /* flags */
#define IO_SYSINIT   0x00000001       /* system is initialized */
#define IO_SYSTSO    0x00000002       /* Foreground run */
#define IO_SYSTBV    0x00000004       /*  */
#define IO_SYSDUMP   0x00000008       /*  */
#define IO_SYSREPORT 0x00000010       /*  */
//#define IO_SYSGO     0x00000020       /*  */
#define IO_SYSEXIT   0x00000040       /*  */
#define IO_SYSSWAP   0x00000080       /* data swap needed */

#define IO_SYSDEBUG  IO_DEBUG         /* global debug */

#define IO_SYSEDAS   0x00000100       /* EDAS data file active */
#define IO_SYSGOOSY  0x00000200       /* GOOSY data file active */
#define IO_SYSUMAC   0x00000400       /* umac data file active */
#define IO_SYSONLINE 0x00000800       /* online event service */

//#define IO_SYSPENDING 0x00000400      /* I/O in progress */
#define IO_SYSNOCONF 0x40000000       /* suppress confirmation */				      

                                      /* here's the main structure */
struct SIO {
   unsigned long ulFlag;              /* global system status flag */
   unsigned long ulGo;                /* 1/0 system is in GO/HALT status */
   unsigned long ulPending;           /* 1/0 I/O pending/clean */
   char *pcDSN;                       /* input dataset name */
   void *pvBuf;                       /* data I/O buffer */
   int mBuf;                          /* allocated length of buffer */     
   FILE *fp;
   long lLastBufNo;                   /* for security: ensure increasing buffer numbers */
                                      /* multi-threading def.s */
   pthread_mutex_t MutexCompletion;
   pthread_cond_t CondCompletion;
   pthread_mutex_t Mutex;
   pthread_cond_t Cond;
   pthread_t Thread;
   struct SIOREV sIR;
   struct SIOFH sFH;
};
                                      /* function prototypes */
void *_ioproc ( void *);				      
void  _iohalt ( void  );
int   _ioclose( void  );
void  _iogo   ( void  );
int   _ioopen ( const char * );
int   _ioread ( void  );

int _init(void);
int _listinit(void);
int _list (void *, long, long);
int _list7(void *, long, long);

                                      /* command processors */
void _iocmdinp( const char *, const char *, const char *, 
                const char *, const char *, const char *,
                const char *,
                long *, long *, long *, long * );
void _iocmdhalt( void );
void _iocmdgo( long *, long *, long * );
void _iocmdwait( long *, long *, long * );
				      
/*
   --- error handling
*/

void _ioemsg( char *, int );

#endif
